@@ -546,15 +546,31 @@ meta_screen_new (MetaDisplay *display,
   screen->trans_picture = None;
   
   {
+    XFontStruct *font_info;
     XGCValues gc_values;
+    gulong value_mask = 0;
     
     gc_values.subwindow_mode = IncludeInferiors;
+    value_mask |= GCSubwindowMode;
     gc_values.function = GXinvert;
+    value_mask |= GCFunction;
     gc_values.line_width = META_WIREFRAME_XOR_LINE_WIDTH;
-    
+    value_mask |= GCLineWidth;
+
+    font_info = XLoadQueryFont (screen->display->xdisplay, "fixed");
+
+    if (font_info != NULL)
+      {
+        gc_values.font = font_info->fid;
+        value_mask |= GCFont;
+        XFreeFontInfo (NULL, font_info, 1);
+      }
+    else
+      meta_warning ("xserver doesn't have 'fixed' font.\n");
+
     screen->root_xor_gc = XCreateGC (screen->display->xdisplay,
                                      screen->xroot,
-                                     GCSubwindowMode | GCFunction | GCLineWidth,
+                                     value_mask,
                                      &gc_values);
   }
   
@@ -655,6 +671,7 @@ void
 meta_screen_free (MetaScreen *screen)
 {
   MetaDisplay *display;
+  XGCValues gc_values = { 0 };
 
   display = screen->display;
 
@@ -705,6 +722,16 @@ meta_screen_free (MetaScreen *screen)
   if (screen->work_area_idle != 0)
     g_source_remove (screen->work_area_idle);
 
+
+  if (XGetGCValues (screen->display->xdisplay,
+                    screen->root_xor_gc,
+                    GCFont,
+                    &gc_values))
+    {
+      XUnloadFont (screen->display->xdisplay,
+                   gc_values.font);
+    }
+
   XFreeGC (screen->display->xdisplay,
            screen->root_xor_gc);
   
